import { useMemo } from 'react';
import { useSyncExternalStore } from 'use-sync-external-store';

function createStore(createState) {
  let state;
  const listeners = new Set();
  const setState = (partial, replace) => {
    const nextState = typeof partial === "function" ? partial(state) : partial;
    if (nextState !== state) {
      const previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach((listener) => listener(state, previousState));
    }
  };
  const getState = () => state;
  const subscribeWithSelector = (listener, selector = getState, equalityFn = Object.is) => {
    let currentSlice = selector(state);
    function listenerToAdd() {
      const nextSlice = selector(state);
      if (!equalityFn(currentSlice, nextSlice)) {
        const previousSlice = currentSlice;
        listener(currentSlice = nextSlice, previousSlice);
      }
    }
    listeners.add(listenerToAdd);
    return () => listeners.delete(listenerToAdd);
  };
  const subscribe = (listener, selector, equalityFn) => {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }
    listeners.add(listener);
    return () => listeners.delete(listener);
  };
  const destroy = () => listeners.clear();
  const api = { setState, getState, subscribe, destroy };
  state = createState(setState, getState, api);
  return api;
}

function useStore(api, selector = api.getState, equalityFn = Object.is) {
  const getSnapshot = useMemo(() => {
    let lastSnapshot;
    let lastSlice;
    return () => {
      let slice = lastSlice;
      const snapshot = api.getState();
      if (lastSnapshot === void 0 || !Object.is(lastSnapshot, snapshot)) {
        slice = selector(snapshot);
        if (lastSlice === void 0 || !equalityFn(lastSlice, slice)) {
          lastSlice = slice;
        } else {
          slice = lastSlice;
        }
        lastSnapshot = snapshot;
      }
      return slice;
    };
  }, [api, selector, equalityFn]);
  return useSyncExternalStore(api.subscribe, getSnapshot);
}
function create(createState) {
  const api = typeof createState === "function" ? createStore(createState) : createState;
  const useBoundStore = (selector, equalityFn) => useStore(api, selector, equalityFn);
  Object.assign(useBoundStore, api);
  return useBoundStore;
}

export { createStore, create as default, useStore };
